﻿///
///  visimport - Visio network mapping for Nmap and Nessus 
///
///
///  Example nmap discovery commandline: nmap -sSUV --top-ports=15 -O --version-light --script=ms-sql-info,mysql-info,nbstat,smb-os-discovery,snmp-brute,snmp-sysdescr -oX Desktop/10.10.10.0.xml 10.10.10.0/24
///
///

using System;
using System.IO;
using System.Net;
using System.Collections.Generic;
using System.Xml;
using System.Linq;
using System.Xml.Linq;
using System.Xml.XPath;
using System.Text;

using Visio = Microsoft.Office.Interop.Visio;

namespace VisImport
{
    class Program
    {
        static void Main(string[] args)
        {
            // this can be overridden by args
            double dropItemsPerRow = 8;

            // location to start drawing shapes
            double dropLocationX = 0;
            double dropLocationY = 10;

            // define constants
            const double dropSpacingX = 1.3;
            const double dropSpacingY = 0.1;
            const double dropSpacingRow = 1.3;
            const double shapeTextSize = 10;

            //const string templateNameU = "NETW_M.VST";
            const string templateNameU = "DTLNET_M.VST";
            const string stencilNameU = "PERIPH_U.VSS";
            const string masterNetworkNameU = "Ethernet";
            const string masterServerNameU = "Server";
            const string masterPrinterNameU = "Printer";
            const string masterSwitchNameU = "Switch";
            const string masterRouterNameU = "Router";
            const string masterFirewallNameU = "Router";

            const string macAddressLookupServiceURL = "http://isquared.nl/doapp.html?appid=mac2vendor&mac=";

            try
            {
                // Parse arguments
                XElement nmapResults = null;
                XElement nessusReport = null;
                for (int i = 0; i < args.Count(); ++i)
                {
                    if (args[i] == "-x" || args[i] == "-X")
                        nmapResults = XElement.Load(args[++i]);
                    if (args[i] == "-n" || args[i] == "-N")
                        nessusReport = XElement.Load(args[++i]);
                    if (args[i] == "-r" || args[i] == "-R")
                        dropItemsPerRow = Int32.Parse(args[++i]);
                }
                if (null == nmapResults)
                    Console.WriteLine("Usage: \n\nVisImport.exe -x nmap-results.xml [-n scan-results.nessus] [-r items-per-row]\n");

                // Start Visio
                Visio.ApplicationClass visapp = new Visio.ApplicationClass();

                // Load the drawing template
                visapp.Documents.Add(templateNameU);

                // Set the default text font size
                visapp.ActiveDocument.Styles["Normal"].get_CellsSRC((short)Visio.VisSectionIndices.visSectionCharacter,
                                                                    (short)Visio.VisRowIndices.visRowCharacter,
                                                                    (short)Visio.VisCellIndices.visCharacterSize
                                                                    ).set_Result(Visio.VisUnitCodes.visPoints, shapeTextSize);
                
                // Load the stencil.
                Visio.Document stencil = visapp.Documents.OpenEx(stencilNameU,
                    (short)(short)Visio.
                    VisOpenSaveArgs.visOpenRO
                    + (short)Visio.VisOpenSaveArgs.
                    visOpenDocked);

                // Drop the subnet shape on the current page 
                Visio.Shape networkShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterNetworkNameU), dropLocationX + ((dropItemsPerRow + 1) * dropSpacingX), dropLocationY / 2);
                networkShape.get_CellsU("Angle").set_Result(Visio.VisUnitCodes.visAngleUnits, 90);
                networkShape.get_CellsU("TxtAngle").set_Result(Visio.VisUnitCodes.visAngleUnits, 180);
                networkShape.get_CellsU("Width").set_Result(Visio.VisUnitCodes.visCentimeters, 30);


                // get the subnet name from the nmap arguments
                string nmapArgs = nmapResults.XPathSelectElement("/").Attribute("args").Value;
                networkShape.Name = nmapArgs.Substring(nmapArgs.LastIndexOf(' '));
                networkShape.Text = networkShape.Name;

                // create the data graphic master object
                Visio.Master dataGraphicMaster = visapp.ActiveDocument.Masters.AddEx(Microsoft.Office.Interop.Visio.VisMasterTypes.visTypeDataGraphic);
                Visio.Master dataGraphicMasterCopy = dataGraphicMaster.Open();
                dataGraphicMasterCopy.Close();

                // iterate over all hosts in the nmap file
                short index = 0;
                foreach (XElement host in nmapResults.Elements("host"))
                {

                    // set the device default properties
                    string deviceType = "server";
                    string deviceIPAddress = "0.0.0.0";
                    string deviceDNSName = "";
                    string deviceHostName = "";
                    string deviceNetBIOSName = "";
                    string deviceMACAddress = "";
                    string deviceMACAddressVendor = "";
                    string deviceOperatingSystem = "";
                    string deviceOSVendor = "";
                    string deviceSNMPCommunity = "unknown";
                    string deviceSysDescr = "";

                    // import the device properties
                    XElement match;
                    
                    // get IPv4 address
                    if (null != (match = host.XPathSelectElement("address[@addrtype='ipv4']")))
                        deviceIPAddress = match.Attribute("addr").Value;

                    // get MAC address
                    if (null != (match = host.XPathSelectElement("address[@addrtype='mac']")))
                    {
                        deviceMACAddress = match.Attribute("addr").Value;
                        deviceMACAddressVendor = match.Attribute("vendor").Value;
                    }
                    
                    // get DNS name
                    if (null != (match = host.XPathSelectElement("hostnames/hostname[@type='PTR']")))
                        deviceDNSName = match.Attribute("name").Value;

                    // get NetBIOS name table
                    string nbstat = "";
                    if (null != (match = host.XPathSelectElement("hostscript/script[@id='nbstat']")))
                        nbstat = match.Attribute("output").Value;
                    if (nbstat.StartsWith("NetBIOS name:"))
                        deviceNetBIOSName = nbstat.Remove(nbstat.IndexOf(',')).Substring(14);
                    if (deviceMACAddress == "" && nbstat.Contains("NetBIOS MAC:") && !nbstat.Contains("NetBIOS MAC: <unknown>"))
                        deviceMACAddress = nbstat.Substring(nbstat.IndexOf("MAC: ") + 5, 17);

                    // lookup MAC address vendor in online database
                    if (deviceMACAddressVendor == "") try
                    {
                        WebResponse response = WebRequest.Create(macAddressLookupServiceURL + deviceMACAddress).GetResponse();
                        using (StreamReader reader = new StreamReader(response.GetResponseStream(), Encoding.Default))
                        {
                            string content = reader.ReadToEnd();
                            int start = content.ToString().IndexOf("<h3>");
                            int end = content.ToString().IndexOf("</h3>");
                            if (start > 0 && end > 0 && end > start)
                                deviceMACAddressVendor = content.ToString().Substring(start + 4, end - start - 4);
                        }
                    }
                    catch (Exception e) { System.Diagnostics.Debug.WriteLine("Error looking up MAC vendor: " + e.Message + "\n"); }

                    // get OS fingerprint
                    if (null != (match = host.XPathSelectElement("os/osmatch[@name and @accuracy > 90]")))
                        deviceOperatingSystem = match.Attribute("name").Value;

                    // get OS vendor and device class
                    if (null != (match = host.XPathSelectElement("os/osclass[@vendor and @osfamily and @accuracy > 90]")))
                    {
                        if (deviceOperatingSystem == "")
                        {
                            deviceOperatingSystem = match.Attribute("vendor").Value + " " +
                                                    match.Attribute("osfamily").Value;
                            if (null != match.Attribute("osgen"))
                                deviceOperatingSystem += " " + match.Attribute("osgen").Value + " ??";
                        }

                        deviceOSVendor = match.Attribute("vendor").Value;

                        if (null != match.Attribute("type"))
                        {
                            if (match.Attribute("type").Value == "switch")
                                deviceType = "switch";
                            else if (match.Attribute("type").Value == "router")
                                deviceType = "router";
                            else if (match.Attribute("type").Value == "firewall")
                                deviceType = "firewall";
                            else if (match.Attribute("type").Value.StartsWith("print"))
                                deviceType = "printer";
                        }
                    }

                    // get the SMB OS fingerprinting
                    if (null != (match = host.XPathSelectElement("hostscript/script[@id='smb-os-discovery']")))
                    {
                        string OS = match.Attribute("output").Value;
                        if (!OS.Contains("Samba") && !OS.Contains("ERROR"))
                            deviceOperatingSystem = OS.Remove(OS.IndexOf("LAN Manager:"));
                    }

                    // get the SNMP SysDescr string
                    if (null != (match = host.XPathSelectElement("ports/port/script[@id='snmp-sysdescr']")))
                        deviceSysDescr = match.Attribute("output").Value;

                    // get the device type from bannergrabs
                    if (null != (match = host.XPathSelectElement("ports/port/service[@devicetype]")))
                    {
                        deviceType = match.Attribute("devicetype").Value;
                        if (deviceType.StartsWith("print"))
                            deviceType = "printer";
                    }

                    // calculate the next drop location
                    if (index > 0 && index % dropItemsPerRow == 0)
                    {
                        dropLocationX -= dropItemsPerRow * dropSpacingX;
                        dropLocationY -= dropSpacingRow;
                    }
                    dropLocationX += dropSpacingX;
                    dropLocationY -= dropSpacingY;
                    index += 1;

                    // drop the device shape
                    Visio.Shape deviceShape;
                    if (deviceType == "switch")
                        deviceShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterSwitchNameU), dropLocationX, dropLocationY);
                    else if (deviceType == "router")
                        deviceShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterRouterNameU), dropLocationX, dropLocationY);
                    else if (deviceType == "firewall")
                        deviceShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterFirewallNameU), dropLocationX, dropLocationY);
                    else if (deviceType == "printer")
                        deviceShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterPrinterNameU), dropLocationX, dropLocationY);
                    else
                        deviceShape = visapp.ActivePage.Drop(stencil.Masters.get_ItemU(masterServerNameU), dropLocationX, dropLocationY);

                    // set the shape name and IP address
                    deviceShape.Name = deviceIPAddress;
                    deviceShape.get_CellsU("Prop.IPAddress").Formula = "=\"" + deviceShape.Name + "\"";

                    // set the MAC address and vendor if available
                    deviceShape.get_CellsU("Prop.MACAddress").Formula = "=\"" + deviceMACAddress + "\"";
                    if (deviceMACAddressVendor != "")
                    {
                        if (deviceMACAddressVendor.Length > 8 &&
                            (deviceMACAddressVendor.EndsWith("ompany") || deviceMACAddressVendor.EndsWith("omputer")))
                            deviceMACAddressVendor = deviceMACAddressVendor.Remove(deviceMACAddressVendor.Length - 8);
                        deviceShape.get_CellsU("Prop.Manufacturer").Formula = "=\"" + deviceMACAddressVendor + "\"";
                    }

                    // set the Operating System / Equipment Vendor
                    if (deviceType == "server")
                        deviceShape.get_CellsU("Prop.OperatingSystem").Formula = "=\"" + deviceOperatingSystem + "\"";
                    else
                        deviceShape.get_CellsU("Prop.Manufacturer").Formula = "=\"" + deviceOSVendor + "\"";

                    // enumerate and set identified network services
                    foreach (XElement networkService in host.XPathSelectElements("ports/port/service[@product]"))
                    {
                        // get port and protocol
                        XElement portNode = networkService.XPathSelectElement("..");
                        string port = portNode.Attribute("portid").Value + portNode.Attribute("protocol").Value;

                        // add a ShapeData row for the service
                        deviceShape.AddNamedRow((short)Visio.VisSectionIndices.visSectionProp, "Port" + port, 0);
                        string description = networkService.Attribute("product").Value;
                        if (networkService.Attributes("version").Count() > 0)
                            description += " " + networkService.Attribute("version").Value;
                        if (networkService.Attributes("extrainfo").Count() > 0)
                        {
                            // if service is snmp, save community string from extrainfo
                            if (networkService.Attribute("name").Value == "snmp")
                                deviceSNMPCommunity = networkService.Attribute("extrainfo").Value;
                            else
                                description += " " + networkService.Attribute("extrainfo").Value;
                        }
                        deviceShape.get_CellsU("Prop.Port" + port).Formula = "=\"" + description + "\"";

                        // if hostname was discovered, use it
                        if (networkService.Attributes("hostname").Count() > 0)
                            deviceHostName = networkService.Attribute("hostname").Value;
                    }

                    // set the SNMP community string
                    deviceShape.get_CellsU("Prop.CommunityString").Formula = "=\"" + deviceSNMPCommunity + "\"";

                    // set the SNMP sysdescr
                    deviceShape.get_CellsU("Prop.ProductDescription").Formula = "=\"" + deviceSysDescr + "\"";

                    // set the hostname(s)
                    deviceShape.Data1 = deviceHostName;
                    deviceShape.Data2 = deviceNetBIOSName;
                    
                    // prefer DNS for NetworkName property
                    if (deviceDNSName != "")
                        deviceShape.get_CellsU("Prop.NetworkName").Formula = "=\"" + deviceDNSName + "\"";
                    else if (deviceHostName != "")
                        deviceShape.get_CellsU("Prop.NetworkName").Formula = "=\"" + deviceHostName + "\"";
                    else
                        deviceShape.get_CellsU("Prop.NetworkName").Formula = "=\"" + deviceNetBIOSName + "\"";
                    
                    // set hostname and IP address as shape text
                    deviceShape.TextStyle = "Normal";
                    deviceShape.Text = deviceIPAddress + "\n";
                    if (deviceHostName != "")
                        deviceShape.Text += deviceHostName;
                    else if (deviceNetBIOSName != "")
                        deviceShape.Text += deviceNetBIOSName;
                    else
                        deviceShape.Text += deviceDNSName;
                    

                    // enumerate and set vulnerability information
                    if (null != nessusReport)
                    try
                    {
                        Int32 maxSeverity = 0;
                        IEnumerable<XElement> reportHosts = nessusReport.XPathSelectElements("Report/ReportHost[HostName='" + deviceIPAddress + "']");
                        foreach (XElement reportHost in reportHosts)
                        {
                            foreach (XElement reportItem in reportHost.XPathSelectElements("ReportItem[pluginID and pluginName and severity>0]"))
                            {
                                Int32 severity = Int32.Parse(reportItem.Element("severity").Value);
                                if (severity > maxSeverity)
                                    maxSeverity = severity;
                                if (severity > 1)
                                {
                                    string rowName = "VulnID" + reportItem.Element("pluginID").Value;
                                    if (0 != deviceShape.get_CellExistsU("Prop." + rowName, 0))
                                        continue;
                                    deviceShape.AddNamedRow((short)Visio.VisSectionIndices.visSectionProp, rowName, 0);
                                    deviceShape.get_CellsU("Prop." + rowName).Formula = "=\"" + reportItem.Element("pluginName").Value + "\"";
                                }
                            }
                        }
                        deviceShape.AddNamedRow((short)Visio.VisSectionIndices.visSectionProp, "VulnSeverity", 0);
                        deviceShape.get_CellsU("Prop.VulnSeverity").ResultIU = maxSeverity;
                    }
                    catch (Exception e) { Console.WriteLine("Error processing vuln data for " + deviceIPAddress + ": " + e.Message + "\n"); }

                    // add the ShapeData graphic overlay to the device shape
                    deviceShape.DataGraphic = dataGraphicMaster;
                    
                    // connect the device to the network shape
                    deviceShape.AutoConnect(networkShape, Visio.VisAutoConnectDir.visAutoConnectDirNone, networkShape);
                }

                // Resize page to fit drawing
                visapp.ActivePage.ResizeToFitContents();

            }
            catch (Exception err)
            {
                Console.WriteLine(err.Message + "\n" + err.StackTrace);
                Environment.Exit(1);
            }

        }
    }
}
